@@ -1,13 +1,15 @@
+#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
 
 #include <src/EPGScanner.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gee/hashset.h>
+#include <gee/iterable.h>
 #include <gee/iterator.h>
 #include <gee/collection.h>
-#include "src/ChannelList.h"
 #include "src/Channel.h"
 #include "src/Device.h"
+#include "src/ChannelList.h"
 #include "src/Event.h"
 #include "src/Schedule.h"
 
@@ -16,6 +18,7 @@
 
 struct _DVBEPGScannerPrivate {
 	DVBDeviceGroup* _DeviceGroup;
+	GStaticRecMutex __lock_pipeline;
 	GstElement* pipeline;
 	GQueue* channels;
 	guint scan_event_id;
@@ -29,6 +32,7 @@ enum  {
 };
 #define DVB_EPG_SCANNER_CHECK_EIT_INTERVAL 15 * 60
 #define DVB_EPG_SCANNER_WAIT_FOR_EIT_DURATION 10
+#define DVB_EPG_SCANNER_PIPELINE_TEMPLATE "dvbsrc name=dvbsrc adapter=%u frontend=%u pids=0:16:17:18 stats-reporting-interval=0 ! mpegtsparse ! fakesink silent=true"
 static void dvb_epg_scanner_reset (DVBEPGScanner* self);
 static void _dvb_epg_scanner_bus_watch_func_gst_bus_message (GstBus* _sender, GstMessage* message, gpointer self);
 static gboolean _dvb_epg_scanner_scan_new_frequency_gsource_func (gpointer self);
@@ -65,7 +69,7 @@ DVBEPGScanner* dvb_epg_scanner_new (DVBDeviceGroup* device) {
          */
 void dvb_epg_scanner_stop (DVBEPGScanner* self) {
 	g_return_if_fail (self != NULL);
-	g_debug ("EPGScanner.vala:37: Stopping EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
+	g_debug ("EPGScanner.vala:40: Stopping EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
 	/* Remove timed scans */
 	if (self->priv->scan_event_id != 0) {
 		g_source_remove (self->priv->scan_event_id);
@@ -80,14 +84,31 @@ void dvb_epg_scanner_stop (DVBEPGScanner* self) {
 
 
 static void dvb_epg_scanner_reset (DVBEPGScanner* self) {
-	GstElement* _tmp0;
+	DVBChannel* c;
+	DVBChannel* _tmp1;
 	g_return_if_fail (self != NULL);
-	if (self->priv->pipeline != NULL) {
-		gst_element_set_state (self->priv->pipeline, GST_STATE_NULL);
+	g_static_rec_mutex_lock (&self->priv->__lock_pipeline);
+	{
+		if (self->priv->pipeline != NULL) {
+			GstBus* bus;
+			GstElement* _tmp0;
+			bus = gst_element_get_bus (self->priv->pipeline);
+			gst_bus_remove_signal_watch (bus);
+			gst_element_set_state (self->priv->pipeline, GST_STATE_NULL);
+			_tmp0 = NULL;
+			self->priv->pipeline = (_tmp0 = NULL, (self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL)), _tmp0);
+			(bus == NULL) ? NULL : (bus = (gst_object_unref (bus), NULL));
+		}
 	}
-	_tmp0 = NULL;
-	self->priv->pipeline = (_tmp0 = NULL, (self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL)), _tmp0);
+	g_static_rec_mutex_unlock (&self->priv->__lock_pipeline);
+	/* clear doesn't unref for us so we do this instead*/
+	c = NULL;
+	_tmp1 = NULL;
+	while ((c = (_tmp1 = (DVBChannel*) g_queue_pop_head (self->priv->channels), (c == NULL) ? NULL : (c = (g_object_unref (c), NULL)), _tmp1)) != NULL) {
+	}
+	/* Vala unref's Channel instances for us*/
 	g_queue_clear (self->priv->channels);
+	(c == NULL) ? NULL : (c = (g_object_unref (c), NULL));
 }
 
 
@@ -108,11 +129,10 @@ gboolean dvb_epg_scanner_start (DVBEPGScanner* self) {
 	GError * inner_error;
 	GeeHashSet* unique_frequencies;
 	DVBDevice* device;
-	GstBus* bus;
-	gboolean _tmp10;
+	gboolean _tmp7;
 	g_return_val_if_fail (self != NULL, FALSE);
 	inner_error = NULL;
-	g_debug ("EPGScanner.vala:63: Starting EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
+	g_debug ("EPGScanner.vala:77: Starting EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
 	/* TODO scan all channels?*/
 	unique_frequencies = gee_hash_set_new (G_TYPE_UINT, NULL, NULL, g_direct_hash, g_direct_equal);
 	{
@@ -138,45 +158,52 @@ gboolean dvb_epg_scanner_start (DVBEPGScanner* self) {
 		gboolean _tmp1;
 		return (_tmp1 = FALSE, (unique_frequencies == NULL) ? NULL : (unique_frequencies = (g_object_unref (unique_frequencies), NULL)), (device == NULL) ? NULL : (device = (g_object_unref (device), NULL)), _tmp1);
 	}
+	g_static_rec_mutex_lock (&self->priv->__lock_pipeline);
 	{
-		char* _tmp4;
-		char* _tmp3;
-		char* _tmp2;
-		GstElement* _tmp5;
-		GstElement* _tmp6;
-		GstElement* _tmp8;
-		GstElement* _tmp7;
-		_tmp4 = NULL;
-		_tmp3 = NULL;
-		_tmp2 = NULL;
-		_tmp5 = NULL;
-		_tmp6 = (_tmp5 = gst_parse_launch (_tmp4 = g_strconcat (_tmp3 = g_strconcat (_tmp2 = g_strdup_printf ("dvbsrc name=dvbsrc adapter=%u frontend=%u ", dvb_device_get_Adapter (device), dvb_device_get_Frontend (device)), "pids=0:16:17:18 stats-reporting-interval=0 ", NULL), "! mpegtsparse ! fakesink silent=true", NULL), &inner_error), _tmp4 = (g_free (_tmp4), NULL), _tmp3 = (g_free (_tmp3), NULL), _tmp2 = (g_free (_tmp2), NULL), _tmp5);
-		if (inner_error != NULL) {
-			goto __catch3_g_error;
-		}
-		_tmp8 = NULL;
-		_tmp7 = NULL;
-		self->priv->pipeline = (_tmp8 = (_tmp7 = _tmp6, (_tmp7 == NULL) ? NULL : gst_object_ref (_tmp7)), (self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL)), _tmp8);
-	}
-	goto __finally3;
-	__catch3_g_error:
-	{
-		GError * e;
-		e = inner_error;
-		inner_error = NULL;
+		GstBus* bus;
 		{
-			gboolean _tmp9;
-			g_error (e->message);
-			return (_tmp9 = FALSE, (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), (unique_frequencies == NULL) ? NULL : (unique_frequencies = (g_object_unref (unique_frequencies), NULL)), (device == NULL) ? NULL : (device = (g_object_unref (device), NULL)), _tmp9);
+			char* _tmp2;
+			GstElement* _tmp3;
+			GstElement* _tmp4;
+			GstElement* _tmp5;
+			_tmp2 = NULL;
+			_tmp3 = NULL;
+			_tmp4 = (_tmp3 = gst_parse_launch (_tmp2 = g_strdup_printf (DVB_EPG_SCANNER_PIPELINE_TEMPLATE, dvb_device_get_Adapter (device), dvb_device_get_Frontend (device)), &inner_error), _tmp2 = (g_free (_tmp2), NULL), _tmp3);
+			if (inner_error != NULL) {
+				goto __catch3_g_error;
+				goto __finally3;
+			}
+			_tmp5 = NULL;
+			self->priv->pipeline = (_tmp5 = _tmp4, (self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL)), _tmp5);
 		}
+		goto __finally3;
+		__catch3_g_error:
+		{
+			GError * e;
+			e = inner_error;
+			inner_error = NULL;
+			{
+				gboolean _tmp6;
+				g_error ("EPGScanner.vala:97: Could not create pipeline: %s", e->message);
+				return (_tmp6 = FALSE, (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), (unique_frequencies == NULL) ? NULL : (unique_frequencies = (g_object_unref (unique_frequencies), NULL)), (device == NULL) ? NULL : (device = (g_object_unref (device), NULL)), _tmp6);
+			}
+		}
+		__finally3:
+		if (inner_error != NULL) {
+			(unique_frequencies == NULL) ? NULL : (unique_frequencies = (g_object_unref (unique_frequencies), NULL));
+			(device == NULL) ? NULL : (device = (g_object_unref (device), NULL));
+			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+			g_clear_error (&inner_error);
+			return FALSE;
+		}
+		bus = gst_element_get_bus (self->priv->pipeline);
+		gst_bus_add_signal_watch (bus);
+		g_signal_connect_object (bus, "message", (GCallback) _dvb_epg_scanner_bus_watch_func_gst_bus_message, self, 0);
+		(bus == NULL) ? NULL : (bus = (gst_object_unref (bus), NULL));
 	}
-	__finally3:
-	;
-	bus = gst_element_get_bus (self->priv->pipeline);
-	gst_bus_add_signal_watch (bus);
-	g_signal_connect_object (bus, "message", (GCallback) _dvb_epg_scanner_bus_watch_func_gst_bus_message, self, 0);
+	g_static_rec_mutex_unlock (&self->priv->__lock_pipeline);
 	self->priv->scan_event_id = g_timeout_add_seconds ((guint) DVB_EPG_SCANNER_WAIT_FOR_EIT_DURATION, _dvb_epg_scanner_scan_new_frequency_gsource_func, self);
-	return (_tmp10 = FALSE, (unique_frequencies == NULL) ? NULL : (unique_frequencies = (g_object_unref (unique_frequencies), NULL)), (device == NULL) ? NULL : (device = (g_object_unref (device), NULL)), (bus == NULL) ? NULL : (bus = (gst_object_unref (bus), NULL)), _tmp10);
+	return (_tmp7 = FALSE, (unique_frequencies == NULL) ? NULL : (unique_frequencies = (g_object_unref (unique_frequencies), NULL)), (device == NULL) ? NULL : (device = (g_object_unref (device), NULL)), _tmp7);
 }
 
 
@@ -190,12 +217,10 @@ static gboolean _dvb_epg_scanner_start_gsource_func (gpointer self) {
          */
 static gboolean dvb_epg_scanner_scan_new_frequency (DVBEPGScanner* self) {
 	DVBChannel* channel;
-	GstElement* _tmp1;
-	GstElement* dvbsrc;
-	gboolean _tmp2;
+	gboolean _tmp1;
 	g_return_val_if_fail (self != NULL, FALSE);
 	if (g_queue_is_empty (self->priv->channels)) {
-		g_debug ("EPGScanner.vala:105: Finished EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
+		g_debug ("EPGScanner.vala:117: Finished EPG scan for group %u", dvb_device_group_get_Id (self->priv->_DeviceGroup));
 		dvb_epg_scanner_reset (self);
 		/* Time the next iteration*/
 		self->priv->queue_scan_event_id = g_timeout_add_seconds ((guint) DVB_EPG_SCANNER_CHECK_EIT_INTERVAL, _dvb_epg_scanner_start_gsource_func, self);
@@ -203,13 +228,17 @@ static gboolean dvb_epg_scanner_scan_new_frequency (DVBEPGScanner* self) {
 	}
 	channel = (DVBChannel*) g_queue_pop_head (self->priv->channels);
 	dvb_schedule_remove_expired_events (dvb_channel_get_Schedule (channel));
-	/*debug ("Scanning channel %s", channel.to_string ());*/
-	gst_element_set_state (self->priv->pipeline, GST_STATE_READY);
-	_tmp1 = NULL;
-	dvbsrc = (_tmp1 = gst_bin_get_by_name ((GST_BIN (self->priv->pipeline)), "dvbsrc"), (_tmp1 == NULL) ? NULL : gst_object_ref (_tmp1));
-	dvb_channel_setup_dvb_source (channel, dvbsrc);
-	gst_element_set_state (self->priv->pipeline, GST_STATE_PLAYING);
-	return (_tmp2 = TRUE, (channel == NULL) ? NULL : (channel = (g_object_unref (channel), NULL)), (dvbsrc == NULL) ? NULL : (dvbsrc = (gst_object_unref (dvbsrc), NULL)), _tmp2);
+	g_static_rec_mutex_lock (&self->priv->__lock_pipeline);
+	{
+		GstElement* dvbsrc;
+		gst_element_set_state (self->priv->pipeline, GST_STATE_READY);
+		dvbsrc = gst_bin_get_by_name ((GST_BIN (self->priv->pipeline)), "dvbsrc");
+		dvb_channel_setup_dvb_source (channel, dvbsrc);
+		gst_element_set_state (self->priv->pipeline, GST_STATE_PLAYING);
+		(dvbsrc == NULL) ? NULL : (dvbsrc = (gst_object_unref (dvbsrc), NULL));
+	}
+	g_static_rec_mutex_unlock (&self->priv->__lock_pipeline);
+	return (_tmp1 = TRUE, (channel == NULL) ? NULL : (channel = (g_object_unref (channel), NULL)), _tmp1);
 }
 
 
@@ -221,7 +250,7 @@ static void dvb_epg_scanner_bus_watch_func (DVBEPGScanner* self, GstBus* bus, Gs
 		case GST_MESSAGE_ELEMENT:
 		{
 			if (_vala_strcmp0 (gst_structure_get_name (message->structure), "dvb-read-failure") == 0) {
-				g_critical ("EPGScanner.vala:133: Could not read from DVB device");
+				g_critical ("EPGScanner.vala:146: Could not read from DVB device");
 				dvb_epg_scanner_stop (self);
 			} else {
 				if (_vala_strcmp0 (gst_structure_get_name (message->structure), "eit") == 0) {
@@ -246,7 +275,7 @@ static void dvb_epg_scanner_bus_watch_func (DVBEPGScanner* self, GstBus* bus, Gs
 			_tmp0 = NULL;
 			(gst_message_parse_error (message, &_tmp0, &_tmp2), gerror = (_tmp1 = _tmp0, (gerror == NULL) ? NULL : (gerror = (g_error_free (gerror), NULL)), _tmp1));
 			debug = (_tmp3 = _tmp2, debug = (g_free (debug), NULL), _tmp3);
-			g_critical ("EPGScanner.vala:144: %s %s", gerror->message, debug);
+			g_critical ("EPGScanner.vala:157: %s %s", gerror->message, debug);
 			dvb_epg_scanner_stop (self);
 			(gerror == NULL) ? NULL : (gerror = (g_error_free (gerror), NULL));
 			debug = (g_free (debug), NULL);
@@ -301,7 +330,7 @@ void dvb_epg_scanner_on_eit_structure (DVBEPGScanner* self, const GstStructure*
 			sid = dvb_epg_scanner_get_uint_val (structure, "service-id");
 			channel = dvb_channel_list_get (dvb_device_group_get_Channels (self->priv->_DeviceGroup), sid);
 			if (channel == NULL) {
-				g_warning ("EPGScanner.vala:170: Could not find channel %u for this device", sid);
+				g_warning ("EPGScanner.vala:183: Could not find channel %u for this device", sid);
 				(channel == NULL) ? NULL : (channel = (g_object_unref (channel), NULL));
 				return;
 			}
@@ -484,6 +513,7 @@ static GObject * dvb_epg_scanner_constructor (GType type, guint n_construct_prop
 
 static void dvb_epg_scanner_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
 	DVBEPGScanner * self;
+	gpointer boxed;
 	self = DVB_EPG_SCANNER (object);
 	switch (property_id) {
 		case DVB_EPG_SCANNER_DEVICE_GROUP:
@@ -523,6 +553,7 @@ static void dvb_epg_scanner_class_init (DVBEPGScannerClass * klass) {
 
 static void dvb_epg_scanner_instance_init (DVBEPGScanner * self) {
 	self->priv = DVB_EPG_SCANNER_GET_PRIVATE (self);
+	g_static_rec_mutex_init (&self->priv->__lock_pipeline);
 }
 
 
@@ -530,6 +561,7 @@ static void dvb_epg_scanner_finalize (GObject* obj) {
 	DVBEPGScanner * self;
 	self = DVB_EPG_SCANNER (obj);
 	(self->priv->_DeviceGroup == NULL) ? NULL : (self->priv->_DeviceGroup = (g_object_unref (self->priv->_DeviceGroup), NULL));
+	g_static_rec_mutex_free (&self->priv->__lock_pipeline);
 	(self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL));
 	(self->priv->channels == NULL) ? NULL : (self->priv->channels = (g_queue_free (self->priv->channels), NULL));
 	G_OBJECT_CLASS (dvb_epg_scanner_parent_class)->finalize (obj);
