@@ -13,7 +13,6 @@ static void _g_slist_free_dvb_event_audio_component_unref (GSList* self);
 static void _g_slist_free_dvb_event_video_component_unref (GSList* self);
 static void _g_slist_free_dvb_event_teletext_component_unref (GSList* self);
 static gint64 dvb_event_get_end_timestamp (DVBEvent* self);
-static GObject * dvb_event_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
 enum  {
 	DVB_EVENT_AUDIO_COMPONENT_DUMMY_PROPERTY
 };
@@ -52,6 +51,35 @@ static void _g_slist_free_dvb_event_teletext_component_unref (GSList* self) {
 }
 
 
+DVBEvent* dvb_event_construct (GType object_type) {
+	DVBEvent* self;
+	GSList* _tmp0;
+	GSList* _tmp1;
+	GSList* _tmp2;
+	self = (DVBEvent*) g_type_create_instance (object_type);
+	_tmp0 = NULL;
+	self->audio_components = (_tmp0 = NULL, (self->audio_components == NULL) ? NULL : (self->audio_components = (_g_slist_free_dvb_event_audio_component_unref (self->audio_components), NULL)), _tmp0);
+	_tmp1 = NULL;
+	self->video_components = (_tmp1 = NULL, (self->video_components == NULL) ? NULL : (self->video_components = (_g_slist_free_dvb_event_video_component_unref (self->video_components), NULL)), _tmp1);
+	_tmp2 = NULL;
+	self->teletext_components = (_tmp2 = NULL, (self->teletext_components == NULL) ? NULL : (self->teletext_components = (_g_slist_free_dvb_event_teletext_component_unref (self->teletext_components), NULL)), _tmp2);
+	self->year = (guint) 0;
+	self->month = (guint) 0;
+	self->hour = (guint) 0;
+	self->day = (guint) 0;
+	self->minute = (guint) 0;
+	self->second = (guint) 0;
+	self->duration = (guint) 0;
+	self->running_status = DVB_EVENT_RUNNING_STATUS_UNDEFINED;
+	return self;
+}
+
+
+DVBEvent* dvb_event_new (void) {
+	return dvb_event_construct (DVB_TYPE_EVENT);
+}
+
+
 /**
          * Whether the event has started and ended in the past
          */
@@ -128,7 +156,6 @@ struct tm dvb_event_get_local_start_time (DVBEvent* self) {
 	time_t utc_timestamp;
 	struct tm _tmp0 = {0};
 	struct tm local_time;
-	0;
 	/* Initialize time zone and set values*/
 	utc_time = dvb_event_get_utc_start_time (self);
 	utc_timestamp = timegm (&utc_time);
@@ -139,7 +166,6 @@ struct tm dvb_event_get_local_start_time (DVBEvent* self) {
 
 struct tm dvb_event_get_utc_start_time (DVBEvent* self) {
 	struct tm utc_time;
-	0;
 	utc_time = dvb_utils_create_utc_time ((gint) self->year, (gint) self->month, (gint) self->day, (gint) self->hour, (gint) self->minute, (gint) self->second);
 	return utc_time;
 }
@@ -245,53 +271,6 @@ gboolean dvb_event_equal (DVBEvent* event1, DVBEvent* event2) {
 }
 
 
-/**
-     * Represents an EPG event (i.e. a show with all its information)
-     */
-DVBEvent* dvb_event_construct (GType object_type) {
-	DVBEvent* self;
-	self = (DVBEvent*) g_type_create_instance (object_type);
-	return self;
-}
-
-
-DVBEvent* dvb_event_new (void) {
-	return dvb_event_construct (DVB_TYPE_EVENT);
-}
-
-
-static GObject * dvb_event_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
-	GObject * obj;
-	DVBEventClass * klass;
-	GObjectClass * parent_class;
-	DVBEvent * self;
-	klass = DVB_EVENT_CLASS (g_type_class_peek (DVB_TYPE_EVENT));
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
-	self = DVB_EVENT (obj);
-	{
-		GSList* _tmp0;
-		GSList* _tmp1;
-		GSList* _tmp2;
-		_tmp0 = NULL;
-		self->audio_components = (_tmp0 = NULL, (self->audio_components == NULL) ? NULL : (self->audio_components = (_g_slist_free_dvb_event_audio_component_unref (self->audio_components), NULL)), _tmp0);
-		_tmp1 = NULL;
-		self->video_components = (_tmp1 = NULL, (self->video_components == NULL) ? NULL : (self->video_components = (_g_slist_free_dvb_event_video_component_unref (self->video_components), NULL)), _tmp1);
-		_tmp2 = NULL;
-		self->teletext_components = (_tmp2 = NULL, (self->teletext_components == NULL) ? NULL : (self->teletext_components = (_g_slist_free_dvb_event_teletext_component_unref (self->teletext_components), NULL)), _tmp2);
-		self->year = (guint) 0;
-		self->month = (guint) 0;
-		self->hour = (guint) 0;
-		self->day = (guint) 0;
-		self->minute = (guint) 0;
-		self->second = (guint) 0;
-		self->duration = (guint) 0;
-		self->running_status = DVB_EVENT_RUNNING_STATUS_UNDEFINED;
-	}
-	return obj;
-}
-
-
 DVBEventAudioComponent* dvb_event_audio_component_construct (GType object_type) {
 	DVBEventAudioComponent* self;
 	self = (DVBEventAudioComponent*) g_type_create_instance (object_type);
